use palette::Hsl;
use serde::Serialize;
use wasm_bindgen::prelude::wasm_bindgen;

use super::{ColorDifference, Differ};

#[derive(Debug, Clone, Copy, Serialize)]
#[wasm_bindgen]
pub struct HSLDifference {
    pub hue: Differ,
    pub saturation: Differ,
    pub lightness: Differ,
}

impl ColorDifference for Hsl {
    type Difference = HSLDifference;

    fn difference(&self, other: &Self) -> Self::Difference {
        let hue = self.hue.into_positive_degrees() - other.hue.into_positive_degrees();
        let saturation = self.saturation - other.saturation;
        let lightness = self.lightness - other.lightness;

        HSLDifference {
            hue: Differ {
                delta: hue,
                percent: hue / self.hue.into_positive_degrees(),
            },
            saturation: Differ {
                delta: saturation,
                percent: saturation / self.saturation,
            },
            lightness: Differ {
                delta: lightness,
                percent: lightness / self.lightness,
            },
        }
    }
}
