use palette::Srgb;
use serde::Serialize;
use wasm_bindgen::prelude::wasm_bindgen;

use super::{ColorDifference, Differ};

#[derive(Debug, Clone, Copy, Serialize)]
#[wasm_bindgen]
pub struct RGBDifference {
    pub r: Differ,
    pub g: Differ,
    pub b: Differ,
}

impl ColorDifference for Srgb {
    type Difference = RGBDifference;

    fn difference(&self, other: &Self) -> Self::Difference {
        let r = self.red - other.red;
        let g = self.green - other.green;
        let b = self.blue - other.blue;

        RGBDifference {
            r: Differ {
                delta: r,
                percent: r / self.red,
            },
            g: Differ {
                delta: g,
                percent: g / self.green,
            },
            b: Differ {
                delta: b,
                percent: b / self.blue,
            },
        }
    }
}
