import { useMemo } from 'react';
import { HctDiffference } from '../../color_functions/color_module';
import { useColorFunction } from '../../ColorFunctionContext';
import styles from './CompareLayout.module.css';
import { CompareMethodProps } from './share-props';

const defaultCompareResult: HctDiffference = {
  hue: {
    delta: 0,
    percent: 0,
  },
  chroma: {
    delta: 0,
    percent: 0,
  },
  lightness: {
    delta: 0,
    percent: 0,
  },
};

export function HCTCompare({ basic = '000000', compare = '000000' }: CompareMethodProps) {
  const { colorFn } = useColorFunction();
  const differ = useMemo(() => {
    if (!colorFn) {
      return defaultCompareResult;
    }
    try {
      const diff = colorFn.differ_in_hct(basic, compare);
      return diff;
    } catch (e) {
      console.error('[Compare in HCT]', e);
    }
    return defaultCompareResult;
  }, [basic, compare]);

  return (
    <div>
      <h6>Compare in HCT Space</h6>
      <div className={styles.elements}>
        <div className={styles.element}>
          <div className={styles.element_name}>H</div>
          <div className={styles.element_values}>
            <span>{differ.hue.delta.toFixed(2)}</span>
            <span>
              {isNaN(differ.hue.percent) ? '0.00' : (differ.hue.percent * 100).toFixed(2)}%
            </span>
          </div>
        </div>
        <div className={styles.element}>
          <div className={styles.element_name}>C</div>
          <div className={styles.element_values}>
            <span>{differ.chroma.delta.toFixed(2)}</span>
            <span>
              {isNaN(differ.chroma.percent) ? '0.00' : (differ.chroma.percent * 100).toFixed(2)}%
            </span>
          </div>
        </div>
        <div className={styles.element}>
          <div className={styles.element_name}>T</div>
          <div className={styles.element_values}>
            <span>{differ.lightness.delta.toFixed(2)}</span>
            <span>
              {isNaN(differ.lightness.percent)
                ? '0.00'
                : (differ.lightness.percent * 100).toFixed(2)}
              %
            </span>
          </div>
        </div>
      </div>
    </div>
  );
}
