import { useMemo } from 'react';
import { MixReversing } from '../../color_functions/color_module';
import { useColorFunction } from '../../ColorFunctionContext';
import styles from './CompareLayout.module.css';
import { CompareMethodProps } from './share-props';

const defaultMixResult: MixReversing = {
  r_factor: 0,
  g_factor: 0,
  b_factor: 0,
  average: 0,
};

export function TintScale({ basic = '000000', compare = '000000' }: CompareMethodProps) {
  const { colorFn } = useColorFunction();
  const mixFactors = useMemo(() => {
    if (!colorFn) {
      return defaultMixResult;
    }
    try {
      const factor = colorFn.tint_scale(basic, compare);
      return factor;
    } catch (e) {
      console.error('[Reversing Tint]', e);
    }
    return defaultMixResult;
  }, [basic, compare]);

  return (
    <div>
      <h6>Compare in RGB Tint.</h6>
      <div className={styles.elements}>
        <div className={styles.element}>
          <div className={styles.element_name}>R</div>
          <div className={styles.element_value}>{(mixFactors.r_factor * 100).toFixed(2)}%</div>
        </div>
        <div className={styles.element}>
          <div className={styles.element_name}>G</div>
          <div className={styles.element_value}>{(mixFactors.g_factor * 100).toFixed(2)}%</div>
        </div>
        <div className={styles.element}>
          <div className={styles.element_name}>B</div>
          <div className={styles.element_value}>{(mixFactors.b_factor * 100).toFixed(2)}%</div>
        </div>
      </div>
      <div className={styles.elements}>
        <div className={styles.element}>
          <div className={styles.element_name}>Average</div>
          <div className={styles.element_value}>{(mixFactors.average * 100).toFixed(2)}%</div>
        </div>
      </div>
    </div>
  );
}
